<?php

/**
 *  This file is part of devMX TeamSpeak3 Webviewer for Wordpress.
 *  Copyright (C) 2012  Maximilian Narr
 *
 *  devMX TeamSpeak3 Webviewer for Wordpress is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  TeamSpeak3 Webviewer is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with devMX TeamSpeak3 Webviewer for Wordpress.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace devmx\Wordpress\TSViewer;

class TSViewer
{

    const VERSION = "1.1.2";
    const VERSION_OPTION_NAME = "tswebviewer_version";
    const TABLE_NAME = "tswebviewer";
    const MAIN_MENU_NAME = "tswv-menu";
    const CONFIG_SUB_MENU_NAME = "tswv-menu-config";
    const OPTIONS_SUB_MENU_PAGE = "tswv-menu-options";
    const FORM_NAME = "tswv-edit";
    const FORM_HIDDEN_FIELD_NAME = "tswv-hidden";
    const TEXT_DOMAIN = "tswv";

    public $pluginPath;
    public $pluginUrl;
    public $cachePath;
    public $adminUrl;
    public $wpdb;
    protected $tableName;

    public function __construct($pluginPath, $pluginUrl, $wpdb)
    {
        $this->wpdb = $wpdb;
        $this->pluginPath = $pluginPath . "/";
        $this->pluginUrl = $pluginUrl;
        $this->cachePath = ABSPATH . 'wp-content/uploads/tswebviewer';
        $this->adminUrl = admin_url();
        $this->tableName = $this->wpdb->prefix . self::TABLE_NAME;
        $this->registerHooks();
        $this->registerShortcodes();
    }

    /**
     * Registers hooks
     * @author Maximilian Narr
     * @since 1.0
     */
    private function registerHooks()
    {
        register_activation_hook($this->pluginPath . "TSViewer.php", array(&$this, 'install'));
        register_deactivation_hook($this->pluginPath .  "TSViewer.php", array(&$this, 'uninstall'));
        add_action('init', array($this, 'setL10n'));
        add_action('admin_menu', array(&$this, 'registerMenus'));
        add_action('admin_notices', array(&$this, 'handleCacheDir'));
        add_action('plugins_loaded', array(&$this, 'upgrade'));

        add_action('widgets_init', function()
                {
                   return register_widget('devmx\Wordpress\TSViewer\TSViewerWidget');
                });
    }

    private function registerShortcodes()
    {
        add_shortcode('tswv', array(&$this, 'renderShortcode'));
        add_shortcode('tswebviewer', array(&$this, 'renderShortcode'));
    }

    /**
     * Registers admin menus
     * @author Maximilian Narr
     * @since 1.0 
     */
    public function registerMenus()
    {
        add_menu_page(__('TeamSpeak3 Webviewer', 'tswv'), __('TS3 Webviewer', 'tswv'), 'manage_options', self::MAIN_MENU_NAME, array(&$this, 'mainMenu'));

        add_submenu_page(self::MAIN_MENU_NAME, __('Add configuration', 'tswv'), __('Add configuration', 'tswv'), 'manage_options', self::CONFIG_SUB_MENU_NAME, array(&$this, 'configMenu'));
    }

    /**
     * Outputs the main menu which lists all config files
     * @author Maximilian Narr
     * @since 1.0 
     */
    public function mainMenu()
    {
        $configDeleted = false;

        // Check if configs need to be deleted
        if (isset($_POST['delete']))
        {
            foreach ($_POST['delete'] as $configId)
            {
                $this->deleteConfiguration($configId);
                $configDeleted = true;
            }
        }

        $configs = $this->getConfigurations();
        $adminURL = $this->adminUrl;
        $menuName = self::CONFIG_SUB_MENU_NAME;

        require_once $this->pluginPath . "tpl/adminMenu.php";
    }

    /**
     * Outputs the edit menu
     * @author Maximilian Narr
     * @since 1.0 
     */
    public function configMenu()
    {
        if (isset($_GET['id'])) $configId = $_GET['id'];


        // Standard config
        $config = array("id" => "",
            "configname" => "",
            "serveradress" => "",
            "serverport" => "",
            "queryport" => "",
            "username" => "",
            "password" => "",
            "downloadIcons" => 1,
            "countryIcons" => 0);

        $configAdded = false;
        $configUpdated = false;

        // If a configuration should be updated or added
        if (isset($_POST[self::FORM_HIDDEN_FIELD_NAME]) && $_POST[self::FORM_HIDDEN_FIELD_NAME] == "Y")
        {
            // If a configfile should be updated
            if (isset($_POST['config-id']) && $_POST['config-id'] != "")
            {
                $this->updateConfiguration($_POST['config-id'], $_POST['serveradress'], $_POST['serverport'], $_POST['queryport'], $_POST['username'], $_POST['password'], $_POST['configname'], $_POST['downloadIcons'], $_POST['countryIcons']);
                $configId = $_POST['config-id'];
                $configUpdated = true;
            }
            // If a configfile should be added
            else
            {
                $configId = $this->addConfiguration($_POST['serveradress'], $_POST['serverport'], $_POST['queryport'], $_POST['username'], $_POST['password'], $_POST['configname'], $_POST['downloadIcons'], $_POST['countryIcons']);
                $configAdded = true;
            }
        }

        // If a configfile should be edited or a new one should be added
        if (isset($configId))
        {
            $id = (int) $configId;
            $data = $this->getConfiguration($id);

            foreach ($data[0] as $key => $value) $config[$key] = $value;
        }

        $formName = self::FORM_NAME;
        $hiddenFieldName = self::FORM_HIDDEN_FIELD_NAME;

        require_once $this->pluginPath . "tpl/configMenu.php";
    }

    /**
     * Renders the shortcodes [tswv] and [tswebviewer]
     * @param array $atts Attributes of the shortcode
     * @return string Formatted HTML for output
     * @since 1.0
     * @author Maximilian Narr
     */
    public function renderShortcode($atts)
    {
        if (isset($atts['id']))
        {
            $id = (int) $atts['id'];

            $configuration = $this->getConfiguration($id);

            if (!count($configuration) == 0) return $this->renderViewer($configuration[0]);
        }
    }

    /**
     * Renders the webviewer for the shortcodes
     * @param array $configuration Configuration to render the server 
     * @return string Formatted HTML for output
     * @since 1.0
     * @author Maximilian Narr
     */
    private function renderViewer($configuration)
    {
        $options = new \devmx\TSWebViewer\RenderOptions();
        $options->stylesheetURL($this->pluginUrl . "css/style.css");
        $options->imgPath($this->pluginUrl . "img");
        $options->connectLink(true);

        $cacheHandler = new \devmx\TSWebViewer\Caching\FileCache($this->cachePath, 180);

        $options->imageCaching(true);
        $options->imageCachingPathPublic(site_url() . "/wp-content/uploads/tswebviewer");
        $options->ImageCachingHandler($cacheHandler);
        $options->HTMLCaching(true);
        $options->HTMLCachingHandler($cacheHandler);

        // Download custom icons
        if ($configuration['downloadIcons'] == "0") $options->downloadCustomImages(false);
        else $options->downloadCustomImages(true);

        // Country icons
        if ($configuration['countryIcons'] == "1")
        {
            $options->countryIconsFileType("png");
            $options->countryIconsPath($this->pluginPath . "img/countries");
            $options->countryIconsUrl($this->pluginUrl . "img/countries");
            $options->showCountryIcons(true);
        }
        else
        {
            $options->showCountryIcons(false);
        }

        if ($configuration['username'] == "" && $configuration['password'] == "") $viewer = new \devmx\TSWebViewer\TSWebViewer($configuration['serveradress'], $configuration['queryport'], $configuration['serverport']);
        else $viewer = new \devmx\TSWebViewer\TSWebViewer($configuration['serveradress'], $configuration['queryport'], $configuration['serverport'], $configuration['username'], $configuration['password']);

        try
        {
            return $viewer->renderServer($options);
        }
        catch (Exception $ex)
        {
            return sprintf("<strong>Fatal Error:</strong> %s<br><strong>line: </strong>%s<br><strong>File: </strong>%s<br><strong>Trace: </strong><pre>%s</pre>", $ex->getMessage(), $ex->getLine(), $ex->getFile(), $ex->getTraceAsString());
        }
    }

    /**
     * Checks if the caching path exists and if it is writable. 
     * Tries to create it if it does not exist yet.
     * Displays Errors if the directory cannot be created or is not writable
     * @since 1.0
     * @author Maximilian Narr 
     */
    public function handleCacheDir()
    {
        $cachePath = $this->cachePath;
        if (is_dir($this->cachePath))
        {
            if (!is_writable($this->cachePath)) require_once $this->pluginPath . "tpl/writeError.php";
        }

        $result = wp_mkdir_p($this->cachePath);

        if (!$result) require_once $this->pluginPath . "tpl/dirError.php";
    }

    /**
     * Loads the language files
     * @since 1.0
     * @author Maximilian Narr 
     */
    public function setL10n()
    {
        $path = basename($this->pluginPath) . "/l10n";
        load_plugin_textdomain(self::TEXT_DOMAIN, false, $path);
    }

    /**
     * Inserts the database for the TeamSpeak3 Webviewer Data
     * @since 1.0
     * @author Maximilian Narr 
     */
    public function install()
    {
        $wpdb = $this->wpdb;

        $sql = "CREATE TABLE " . $this->tableName . " (
            id INT(5) NOT NULL AUTO_INCREMENT,
            configname VARCHAR(60) NOT NULL,
            serveradress VARCHAR(40) NOT NULL,
            serverport INT(6) NOT NULL,
            queryport INT(6) NOT NULL,
            username VARCHAR(30) NOT NULL,
            password VARCHAR(60) NOT NULL,
            PRIMARY KEY  (id)
            );";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        dbDelta($sql);
        add_option(self::VERSION_OPTION_NAME, "1.0");
        $this->upgrade();
    }
    
    /**
     * Deletes the version option 
     * @since 1.1.2
     * @author Maximilian Narr
     */
    public function uninstall()
    {
        delete_option(self::VERSION_OPTION_NAME);
    }

    /**
     * Upgrades the table structure to the latest version
     * @author Maximilian Narr
     * @since 1.0 
     */
    public function upgrade()
    {
        $wpdb = $this->wpdb;

        // Upgrade to database version 1.1
        if ( version_compare(get_option(self::VERSION_OPTION_NAME), "1.1", "<"))
        {
            $sql = "CREATE TABLE " . $this->tableName . " (
            id INT(5) NOT NULL AUTO_INCREMENT,
            configname VARCHAR(60) NOT NULL,
            serveradress VARCHAR(40) NOT NULL,
            serverport INT(6) NOT NULL,
            queryport INT(6) NOT NULL,
            username VARCHAR(30) NOT NULL,
            password VARCHAR(60) NOT NULL,
            downloadIcons BOOLEAN NOT NULL DEFAULT 1,
            countryIcons BOOLEAN NOT NULL DEFAULT 0,
            PRIMARY KEY  (id)
            );";

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            dbDelta($sql);

            update_option(self::VERSION_OPTION_NAME, "1.1");
        }
    }

    /**
     * Returns an array containing all available webviewer configuratons
     * @return array All available webviewer configurations
     * @author Maximilian Narr
     * @since 1.0
     */
    private function getConfigurations()
    {
        $sql = "SELECT * FROM " . $this->tableName;

        return $this->wpdb->get_results($sql, ARRAY_A);
    }

    /**
     * Returns the requested configuration with id $id
     * @param int $id Id of the configuration
     * @return array Options of configuration with id $id 
     * @author Maximilian Narr
     * @since 1.0
     */
    private function getConfiguration($id)
    {
        $sql = "SELECT * FROM " . $this->tableName . " WHERE id = %d";
        $sql = $this->wpdb->prepare($sql, $id);
        return $this->wpdb->get_results($sql, ARRAY_A);
    }

    /**
     * Adds a new configuration to the databse
     * @param string $serveradress Adress to the server
     * @param int $serverport Port of the server
     * @param int $queryport Queryport of the server
     * @param string $username Username to login to query
     * @param string $password Password of the user to login to query
     * @param string $configname Name of the configuration
     * @param bool $downloadIcons If custom icons should be downloaded
     * @param bool $countryIcons If countryicons should be shown
     * @return int Id of the inserted row
     * @author Maximilian Narr
     * @since 1.0
     */
    private function addConfiguration($serveradress, $serverport, $queryport, $username, $password, $configname, $downloadIcons, $countryIcons)
    {
        $sql = "INSERT INTO " . $this->tableName . " (serveradress, serverport, queryport, username, password, configname, downloadIcons, countryIcons) VALUES (%s, %d, %d, %s, %s, %s, %d, %d);";
        $sql = $this->wpdb->prepare($sql, $serveradress, $serverport, $queryport, $username, $password, $configname, $downloadIcons, $countryIcons);
        $this->wpdb->query($sql);
        return $this->wpdb->insert_id;
    }

    /**
     * Updates a configuration
     * @param int $id Id of the configuration
     * @param string $serveradress Adress to the server
     * @param int $serverport Port of the server
     * @param int $queryport Queryport of the server
     * @param string $username Username to login to query
     * @param string $password Password of the user to login to query
     * @param string $configname Name of the configuration
     * @param bool $downloadIcons If custom icons should be downloaded
     * @param bool $countryIcons If countryicons should be shown
     * @return int
     * @author Maximilian Narr
     * @since 1.0
     */
    private function updateConfiguration($id, $serveradress, $serverport, $queryport, $username, $password, $configname, $downloadIcons, $countryIcons)
    {
        $sql = "UPDATE " . $this->tableName . " SET serveradress = %s, serverport = %d, queryport = %d, username = %s, password = %s, configname = %s, downloadIcons = %d, countryIcons = %d WHERE id = %d;";
        $sql = $this->wpdb->prepare($sql, $serveradress, $serverport, $queryport, $username, $password, $configname, $downloadIcons, $countryIcons, $id);
        return $this->wpdb->query($sql);
    }

    /**
     * Deleted the configuration with the id $id from the databse
     * @param int $id Id of the configuration to delete
     * @return int
     * @author Maximilian Narr
     * @since 1.0
     */
    private function deleteConfiguration($id)
    {
        $sql = "DELETE FROM " . $this->tableName . " WHERE id = %d";
        $sql = $this->wpdb->prepare($sql, $id);
        return $this->wpdb->query($sql);
    }

}

?>
